@@ -369,8 +369,8 @@ const struct prefs vars[] = {
 	{"away_size_max", P_OFFINT (away_size_max), TYPE_INT},
 	{"away_timeout", P_OFFINT (away_timeout), TYPE_INT},
 
+	{"completion_amount", P_OFFINT (completion_amount), TYPE_INT},
 	{"completion_auto", P_OFFINT (nickcompletion), TYPE_BOOL},
-	{"completion_old", P_OFFINT (old_nickcompletion), TYPE_BOOL},
 	{"completion_suffix", P_OFFSET (nick_suffix), TYPE_STR},
 
 	{"dcc_auto_chat", P_OFFINT (autodccchat), TYPE_INT},
@@ -401,9 +401,6 @@ const struct prefs vars[] = {
 	{"flood_ctcp_time", P_OFFINT (ctcp_time_limit), TYPE_INT},
 	{"flood_msg_num", P_OFFINT (msg_number_limit), TYPE_INT},
 	{"flood_msg_time", P_OFFINT (msg_time_limit), TYPE_INT},
-#ifdef USE_ZVT
-	{"font_shell", P_OFFSET (font_shell), TYPE_STR},
-#endif
 
 	{"gui_auto_open_chat", P_OFFINT (autoopendccchatwindow), TYPE_BOOL},
 	{"gui_auto_open_dialog", P_OFFINT (autodialog), TYPE_BOOL},
@@ -434,13 +431,10 @@ const struct prefs vars[] = {
 	{"gui_win_height", P_OFFINT (mainwindow_height), TYPE_INT},
 	{"gui_win_left", P_OFFINT (mainwindow_left), TYPE_INT},
 	{"gui_win_save", P_OFFINT (mainwindow_save), TYPE_BOOL},
+	{"gui_win_state", P_OFFINT (gui_win_state), TYPE_INT},
 	{"gui_win_top", P_OFFINT (mainwindow_top), TYPE_INT},
 	{"gui_win_width", P_OFFINT (mainwindow_width), TYPE_INT},
 
-#ifdef USE_HEBREW
-	{"hebrew", P_OFFINT (hebrew), TYPE_BOOL},
-#endif
-
 #ifdef WIN32
 	{"identd", P_OFFINT (identd), TYPE_BOOL},
 #endif
@@ -518,6 +512,7 @@ const struct prefs vars[] = {
 	{"text_indent", P_OFFINT (indent_nicks), TYPE_BOOL},
 	{"text_max_indent", P_OFFINT (max_auto_indent), TYPE_INT},
 	{"text_max_lines", P_OFFINT (max_lines), TYPE_INT},
+	{"text_show_marker", P_OFFINT (show_marker), TYPE_BOOL},
 	{"text_show_sep", P_OFFINT (show_separator), TYPE_BOOL},
 	{"text_stripcolor", P_OFFINT (stripcolor), TYPE_BOOL},
 	{"text_thin_sep", P_OFFINT (thin_separator), TYPE_BOOL},
@@ -573,6 +568,7 @@ load_config (void)
 	memset (&prefs, 0, sizeof (struct xchatprefs));
 
 	/* put in default values, anything left out is automatically zero */
+	prefs.completion_amount = 5;
 	prefs.away_timeout = 60;
 	prefs.away_size_max = 300;
 	prefs.timestamp_logs = 1;
@@ -860,10 +856,18 @@ int
 cmd_set (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	int wild = FALSE;
+	int quiet = FALSE;
 	int i = 0, finds = 0, found;
 	char *var = word[2];
 	char *val = word_eol[3];
 
+	if (strcasecmp (word[2], "-quiet") == 0)
+	{
+		var = word[3];
+		val = word_eol[4];
+		quiet = TRUE;
+	}
+
 	if (!*var)
 	{
 		set_list (sess, tbuf);
@@ -893,7 +897,8 @@ cmd_set (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 				{
 					strncpy ((char *) &prefs + vars[i].offset, val, vars[i].len);
 					((char *) &prefs)[vars[i].offset + vars[i].len - 1] = 0;
-					PrintTextf (sess, "%s set to: %s\n", var, (char *) &prefs + vars[i].offset);
+					if (!quiet)
+						PrintTextf (sess, "%s set to: %s\n", var, (char *) &prefs + vars[i].offset);
 				} else
 				{
 					set_showval (sess, &vars[i], tbuf);
@@ -917,8 +922,9 @@ cmd_set (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 					{
 						*((int *) &prefs + vars[i].offset) = atoi (val);
 					}
-					PrintTextf (sess, "%s set to: %d\n", var,
-									*((int *) &prefs + vars[i].offset));
+					if (!quiet)
+						PrintTextf (sess, "%s set to: %d\n", var,
+										*((int *) &prefs + vars[i].offset));
 				} else
 				{
 					set_showval (sess, &vars[i], tbuf);
@@ -930,7 +936,7 @@ cmd_set (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	}
 	while (vars[i].name);
 
-	if (!finds)
+	if (!finds && !quiet)
 		PrintText (sess, "No such variable.\n");
 
 	return TRUE;
