@@ -715,8 +715,6 @@ dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 							 dcc->file, dcc->destfile, dcc->nick, buf, 0);
 			return TRUE;
 		}
-
-		dcc_calc_cps (dcc);
 	}
 }
 
@@ -908,7 +906,6 @@ abortit:
 	{
 		dcc->pos += sent;
 		dcc->lasttime = time (0);
-		dcc_calc_cps (dcc);
 	}
 
 	/* have we sent it all yet? */
@@ -1218,7 +1215,7 @@ dcc_send (struct session *sess, char *to, char *file, int maxcps, int passive)
 		{
 			if (st.st_size > 0)
 			{
-				dcc->offertime = time (0);
+				dcc->starttime = dcc->offertime = time (0);
 				dcc->serv = sess->server;
 				dcc->dccstat = STAT_QUEUED;
 				dcc->size = st.st_size;
@@ -1622,7 +1619,7 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 	if (!strcasecmp (type, "CHAT"))
 	{
 		port = atoi (word[8]);
-		sscanf (word[7], "%lu", &addr);
+		addr = strtoul (word[7], NULL, 10);
 
 		if (port == 0)
 			pasvid = atoi (word[9]);
@@ -1690,7 +1687,7 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 			dcc = find_dcc (nick, word[6], TYPE_SEND);
 		if (dcc)
 		{
-			sscanf (word[8], "%lu", &size);
+			size = strtoul (word[8], NULL, 10);
 			dcc->resumable = size;
 			if (dcc->resumable < dcc->size)
 			{
@@ -1734,8 +1731,8 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 		int psend = 0;
 
 		port = atoi (word[8]);
-		sscanf (word[7], "%lu", &addr);
-		sscanf (word[9], "%lu", &size);
+		addr = strtoul (word[7], NULL, 10);
+		size = strtoul (word[9], NULL, 10);
 
 		if (port == 0) /* Passive dcc requested */
 			pasvid = atoi (word[10]);
