@@ -164,7 +164,7 @@ inbound_privmsg (server *serv, char *from, char *ip, char *text)
 			}
 			set_topic (sess, ip);
 		}
-		inbound_chanmsg (serv, NULL, from, text, FALSE);
+		inbound_chanmsg (serv, NULL, NULL, from, text, FALSE);
 		return;
 	}
 
@@ -204,7 +204,11 @@ SearchNick (char *text, char *nicks)
 		ns = strlen (n);
 		while ((p = nocasestrstr (t, n)))
 		{
-			if ((p == text || !isalnum (*(p - 1))) && !isalnum (*(p + ns)))
+			char *prev_char = (p == text) ? NULL : g_utf8_prev_char (p);
+			char *next_char = p + ns;
+			if ((!prev_char ||
+			     !g_unichar_isalnum (g_utf8_get_char(prev_char))) &&
+			    !g_unichar_isalnum (g_utf8_get_char(next_char)))
 			{
 				free (text);
 				return 1;
@@ -224,11 +228,17 @@ is_hilight (char *text, session *sess, server *serv)
 {
 	if ((SearchNick (text, serv->nick)) || SearchNick (text, prefs.bluestring))
 	{
+#ifdef WIN32
+		if (sess != current_tab)
+			sess->nick_said = TRUE;
+		fe_set_hilight (sess);
+#else
 		if (sess != current_tab)
 		{
 			sess->nick_said = TRUE;
 			fe_set_hilight (sess);
 		}
+#endif
 		return 1;
 	}
 	return 0;
@@ -304,24 +314,26 @@ inbound_action (session *sess, char *chan, char *from, char *text, int fromme)
 }
 
 void
-inbound_chanmsg (server *serv, char *chan, char *from, char *text, char fromme)
+inbound_chanmsg (server *serv, session *sess, char *chan, char *from, char *text, char fromme)
 {
 	struct User *user;
-	session *sess;
 	int hilight = FALSE;
 	char nickchar[2] = "\000";
 
-	if (chan)
-	{
-		sess = find_channel (serv, chan);
-		if (!sess && !is_channel (serv, chan))
-			sess = find_dialog (serv, chan);
-	} else
-	{
-		sess = find_dialog (serv, from);
-	}
 	if (!sess)
-		return;
+	{
+		if (chan)
+		{
+			sess = find_channel (serv, chan);
+			if (!sess && !is_channel (serv, chan))
+				sess = find_dialog (serv, chan);
+		} else
+		{
+			sess = find_dialog (serv, from);
+		}
+		if (!sess)
+			return;
+	}
 
 	if (sess != current_tab)
 	{
@@ -674,7 +686,12 @@ set_server_name (struct server *serv, char *name)
 	if (name[0] == 0)
 		name = serv->hostname;
 
-	safe_strcpy (serv->servername, name, sizeof (serv->servername));
+	/* strncpy parameters must NOT overlap */
+	if (name != serv->servername)
+	{
+		safe_strcpy (serv->servername, name, sizeof (serv->servername));
+	}
+
 	while (list)
 	{
 		sess = (session *) list->data;
@@ -737,7 +754,7 @@ inbound_ping_reply (session *sess, char *timestring, char *from)
 		lag = 1;
 	}
 
-	sscanf (timestring, "%lu", &tim);
+	tim = strtoul (timestring, NULL, 10);
 	nowtim = make_ping_time ();
 	dif = nowtim - tim;
 
@@ -1010,7 +1027,7 @@ set_default_modes (server *serv)
 
 	if (modes[1] != '\0')
 	{
-		serv->p_nick_mode (serv, serv->nick, modes);
+		serv->p_mode (serv, serv->nick, modes);
 	}
 }
 
@@ -1149,6 +1166,15 @@ inbound_banlist (session *sess, time_t stamp, char *chan, char *mask, char *bann
 		fe_add_ban_list (sess, mask, banner, time_str);
 }
 
+/* execute 1 end-of-motd command */
+
+static int
+inbound_exec_eom_cmd (char *str, void *sess)
+{
+	handle_command (sess, (str[0] == '/') ? str + 1 : str, TRUE);
+	return 1;
+}
+
 void
 inbound_login_end (session *sess, char *text)
 {
@@ -1158,12 +1184,15 @@ inbound_login_end (session *sess, char *text)
 	{
 		if (prefs.ip_from_server && serv->use_who)
 		{
-			serv->skip_next_who = TRUE;
-			serv->p_get_ip (serv, serv->nick);	/* sends WHO mynick */
+			serv->skip_next_userhost = TRUE;
+			serv->p_get_ip_uh (serv, serv->nick);	/* sends USERHOST mynick */
 		}
 		set_default_modes (serv);
+
+		/* there may be more than 1, separated by \n */
 		if (serv->eom_cmd)
-			handle_command (sess, serv->eom_cmd, TRUE);
+			token_foreach (serv->eom_cmd, '\n', inbound_exec_eom_cmd, sess);
+
 		check_willjoin_channels (serv);
 		if (serv->supports_watch)
 			notify_send_watches (serv);
