@@ -468,7 +468,7 @@ ban (session * sess, char *tbuf, char *mask, char *bantypestr, int deop)
 	{
 		sprintf (tbuf, "+b %s", mask);
 	}
-	serv->p_chan_mode (serv, sess->channel, tbuf);
+	serv->p_mode (serv, sess->channel, tbuf);
 }
 
 static int
@@ -481,7 +481,7 @@ cmd_ban (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		ban (sess, tbuf, mask, word[3], 0);
 	} else
 	{
-		sess->server->p_chan_mode (sess->server, sess->channel, "+b");	/* banlist */
+		sess->server->p_mode (sess->server, sess->channel, "+b");	/* banlist */
 	}
 
 	return TRUE;
@@ -1950,6 +1950,23 @@ cmd_me (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return TRUE;
 }
 
+static int
+cmd_mode (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	/* +channel channels are dying, let those servers whine about modes.
+	 * return info about current channel if available and no info is given */
+	if ((*word[2] == '+') || (*word[2] == 0) || (!is_channel(sess->server, word[2]) && 
+				!(rfc_casecmp(sess->server->nick, word[2]) == 0)))
+	{
+		if(sess->channel[0] == 0)
+			return FALSE;
+		sess->server->p_mode (sess->server, sess->channel, word_eol[2]);
+	}	
+	else
+		sess->server->p_mode (sess->server, word[2], word_eol[3]);
+	return TRUE;
+}
+
 static int
 mop_cb (struct User *user, multidata *data)
 {
@@ -2018,10 +2035,16 @@ cmd_msg (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 			if (!newsess)
 				newsess = find_channel (sess->server, nick);
 			if (newsess)
-				inbound_chanmsg (newsess->server, newsess->channel,
+				inbound_chanmsg (newsess->server, NULL, newsess->channel,
 									  newsess->server->nick, msg, TRUE);
 			else
+			{
+				/* mask out passwords */
+				if (strcasecmp (nick, "nickserv") == 0 &&
+					 strncasecmp (msg, "identify ", 9) == 0)
+					msg = "identify ****";
 				EMIT_SIGNAL (XP_TE_MSGSEND, sess, nick, msg, NULL, NULL, 0);
+			}
 
 			return TRUE;
 		}
@@ -2547,7 +2570,7 @@ cmd_wallchan (struct session *sess, char *tbuf, char *word[],
 			sess = list->data;
 			if (sess->type == SESS_CHANNEL)
 			{
-				inbound_chanmsg (sess->server, sess->channel,
+				inbound_chanmsg (sess->server, NULL, sess->channel,
 									  sess->server->nick, word_eol[2], TRUE);
 				sess->server->p_message (sess->server, sess->channel, word_eol[2]);
 			}
@@ -2689,6 +2712,7 @@ const struct commands xc_cmds[] = {
 	 N_("ME <action>, sends the action to the current channel (actions are written in the 3rd person, like /me jumps)")},
 	{"MKICK", cmd_mkick, 1, 1,
 	 N_("MKICK, Mass kicks everyone except you in the current channel (needs chanop)")},
+	{"MODE", cmd_mode, 1, 0, 0},
 	{"MOP", cmd_mop, 1, 1,
 	 N_("MOP, Mass op's all users in the current channel (needs chanop)")},
 	{"MSG", cmd_msg, 0, 0, N_("MSG <nick> <message>, sends a private message")},
@@ -2741,7 +2765,7 @@ const struct commands xc_cmds[] = {
 	{"SERVER", cmd_server, 0, 0,
 	 N_("SERVER <host> [<port>] [<password>], connects to a server, the default port is 6667")},
 #endif
-	{"SET", cmd_set, 0, 0, N_("SET <variable> [<value>]")},
+	{"SET", cmd_set, 0, 0, N_("SET [-quiet] <variable> [<value>]")},
 	{"SETTAB", cmd_settab, 0, 0, 0},
 	{"TOPIC", cmd_topic, 1, 1,
 	 N_("TOPIC [<topic>], sets the topic if one is given, else shows the current topic")},
@@ -3196,7 +3220,7 @@ handle_say (session *sess, char *text, int check_spch)
 		dcc = dcc_write_chat (sess->channel, text);
 		if (dcc)
 		{
-			inbound_chanmsg (sess->server, sess->channel,
+			inbound_chanmsg (sess->server, NULL, sess->channel,
 								  sess->server->nick, text, TRUE);
 			set_topic (sess, net_ip (dcc->addr));
 			goto xit;
@@ -3240,7 +3264,7 @@ handle_say (session *sess, char *text, int check_spch)
 			text[max] = 0;			  /* insert a NULL terminator to shorten it */
 		}
 
-		inbound_chanmsg (sess->server, sess->channel, sess->server->nick,
+		inbound_chanmsg (sess->server, sess, sess->channel, sess->server->nick,
 							  text, TRUE);
 		sess->server->p_message (sess->server, sess->channel, text);
 
