@@ -137,15 +137,9 @@ irc_invite (server *serv, char *channel, char *nick)
 }
 
 static void
-irc_chan_mode (server *serv, char *channel, char *mode)
+irc_mode (server *serv, char *target, char *mode)
 {
-	tcp_sendf (serv, "MODE %s %s\r\n", channel, mode);
-}
-
-static void
-irc_nick_mode (server *serv, char *nick, char *mode)
-{
-	tcp_sendf (serv, "MODE %s %s\r\n", nick, mode);
+	tcp_sendf (serv, "MODE %s %s\r\n", target, mode);
 }
 
 /* find channel info when joined */
@@ -164,6 +158,14 @@ irc_user_list (server *serv, char *channel)
 	tcp_sendf (serv, "WHO %s\r\n", channel);
 }
 
+/* userhost */
+
+static void
+irc_userhost (server *serv, char *nick)
+{
+	tcp_sendf (serv, "USERHOST %s\r\n", nick);
+}
+
 static void
 irc_away_status (server *serv, char *channel)
 {
@@ -308,11 +310,11 @@ process_numeric (session * sess, int n,
 		if ((strncmp(word[7], "PTnet", 5) == 0) &&
 			(strncmp(word[8], "IRC", 3) == 0) &&
 			(strncmp(word[9], "Network", 7) == 0) &&
-			(strchr(word[10], '@') != NULL))
+			(strrchr(word[10], '@') != NULL))
 		{
 			serv->use_who = FALSE;
 			if (prefs.ip_from_server)
-				inbound_foundip (sess, strchr(word[10], '@')+1);
+				inbound_foundip (sess, strrchr(word[10], '@')+1);
 		}
 		goto def;
 
@@ -346,6 +348,25 @@ process_numeric (session * sess, int n,
 						(word_eol[5][0] == ':') ? word_eol[5] + 1 : word_eol[5]);
 		break;
 
+	case 302:
+		if (serv->skip_next_userhost)
+		{
+			char *eq = strchr (word[4], '=');
+			if (eq)
+			{
+				*eq = 0;
+				if (!serv->p_cmp (word[4] + 1, serv->nick))
+				{
+					char *at = strrchr (eq + 1, '@');
+					if (at)
+						inbound_foundip (sess, at + 1);
+				}
+			}
+
+			serv->skip_next_userhost = FALSE;
+		}
+		else goto def;
+
 	case 303:
 		word[4]++;
 		notify_markonline (serv, word);
@@ -416,7 +437,7 @@ process_numeric (session * sess, int n,
 
 	case 321:
 		if (!fe_is_chanwindow (sess->server))
-			EMIT_SIGNAL (XP_TE_CHANLISTHEAD, sess, NULL, NULL, NULL, NULL, 0);
+			EMIT_SIGNAL (XP_TE_CHANLISTHEAD, serv->server_session, NULL, NULL, NULL, NULL, 0);
 		break;
 
 	case 322:
@@ -425,14 +446,14 @@ process_numeric (session * sess, int n,
 			fe_add_chan_list (sess->server, word[4], word[5], word_eol[6] + 1);
 		} else
 		{
-			PrintTextf (sess, "%-16s %-7d %s\017\n",
+			PrintTextf (serv->server_session, "%-16s %-7d %s\017\n",
 							word[4], atoi (word[5]), word_eol[6] + 1);
 		}
 		break;
 
 	case 323:
 		if (!fe_is_chanwindow (sess->server))
-			EMIT_SIGNAL (XP_TE_SERVTEXT, sess, text, word[1], NULL, NULL, 0);
+			EMIT_SIGNAL (XP_TE_SERVTEXT, serv->server_session, text, word[1], NULL, NULL, 0);
 		else
 			fe_chan_list_end (sess->server);
 		break;
@@ -495,7 +516,6 @@ process_numeric (session * sess, int n,
 		break;
 
 	case 352:						  /* WHO */
-		if (!serv->skip_next_who)
 		{
 			unsigned int away = 0;
 			session *who_sess = find_channel (serv, word[4]);
@@ -510,10 +530,6 @@ process_numeric (session * sess, int n,
 			if (!who_sess || !who_sess->doing_who)
 				EMIT_SIGNAL (XP_TE_SERVTEXT, serv->server_session, text, word[1],
 								 NULL, NULL, 0);
-		} else
-		{
-			if (!serv->p_cmp (word[8], serv->nick))
-				inbound_foundip (sess, word[6]);
 		}
 		break;
 
@@ -543,10 +559,6 @@ process_numeric (session * sess, int n,
 		break;
 
 	case 315:						  /* END OF WHO */
-		if (serv->skip_next_who)
-		{
-			serv->skip_next_who = FALSE;
-		} else
 		{
 			session *who_sess;
 			who_sess = find_channel (serv, word[4]);
@@ -761,7 +773,7 @@ process_named_msg (session *sess, char *type, char *word[], char *word_eol[])
 				{
 					if (ignore_check (word[1], IG_CHAN))
 						return;
-					inbound_chanmsg (serv, to, nick, text, FALSE);
+					inbound_chanmsg (serv, NULL, to, nick, text, FALSE);
 				} else
 				{
 					if (ignore_check (word[1], IG_PRIV))
@@ -938,13 +950,13 @@ proto_fill_her_up (server *serv)
 	serv->p_join = irc_join;
 	serv->p_login = irc_login;
 	serv->p_join_info = irc_join_info;
-	serv->p_chan_mode = irc_chan_mode;
-	serv->p_nick_mode = irc_nick_mode;
+	serv->p_mode = irc_mode;
 	serv->p_user_list = irc_user_list;
 	serv->p_away_status = irc_away_status;
 	/*serv->p_get_ip = irc_get_ip;*/
 	serv->p_whois = irc_user_whois;
 	serv->p_get_ip = irc_user_list;
+	serv->p_get_ip_uh = irc_userhost;
 	serv->p_set_back = irc_set_back;
 	serv->p_set_away = irc_set_away;
 	serv->p_message = irc_message;
