@@ -1,6 +1,7 @@
 #include "../../config.h"
 
 #include <glib/gslist.h>
+#include <glib/glist.h>
 #include <glib/gutils.h>
 #include <glib/giochannel.h>
 #include <glib/gstrfuncs.h>
@@ -162,6 +163,7 @@ struct xchatprefs
 	int mainwindow_top;
 	int mainwindow_width;
 	int mainwindow_height;
+	int gui_win_state;
 	int dialog_left;
 	int dialog_top;
 	int dialog_width;
@@ -196,7 +198,7 @@ struct xchatprefs
 	unsigned int userlistbuttons;
 	unsigned int showhostname_in_userlist;
 	unsigned int nickcompletion;
-	unsigned int old_nickcompletion;
+	unsigned int completion_amount;
 	unsigned int tabchannels;
 	unsigned int paned_userlist;
 	unsigned int autodccchat;
@@ -239,6 +241,7 @@ struct xchatprefs
 	unsigned int style_inputbox;
 	unsigned int windows_as_tabs;
 	unsigned int indent_nicks;
+	unsigned int show_marker;
 	unsigned int show_separator;
 	unsigned int thin_separator;
 	unsigned int auto_indent;
@@ -344,12 +347,12 @@ typedef struct server
 	void (*p_join)(struct server *, char *channel, char *key);
 	void (*p_login)(struct server *, char *user, char *realname);
 	void (*p_join_info)(struct server *, char *channel);
-	void (*p_chan_mode)(struct server *, char *channel, char *mode);
-	void (*p_nick_mode)(struct server *, char *nick, char *mode);
+	void (*p_mode)(struct server *, char *target, char *mode);
 	void (*p_user_list)(struct server *, char *channel);
 	void (*p_away_status)(struct server *, char *channel);
 	void (*p_whois)(struct server *, char *nicks);
 	void (*p_get_ip)(struct server *, char *nick);
+	void (*p_get_ip_uh)(struct server *, char *nick);
 	void (*p_set_back)(struct server *);
 	void (*p_set_away)(struct server *, char *reason);
 	void (*p_message)(struct server *, char *channel, char *text);
@@ -402,7 +405,8 @@ typedef struct server
 	void *network;						/* for servlistgui.c */
 
 	GSList *outbound_queue;
-	int next_send;						/* cptr->since in ircu */
+	time_t next_send;						/* cptr->since in ircu */
+	time_t prev_now;					/* previous now-time */
 	int sendq_len;						/* queue size */
 
 	struct session *front_session;	/* front-most window/tab */
@@ -427,7 +431,7 @@ typedef struct server
 	unsigned int connected:1;
 	unsigned int connecting:1;
 	int no_login:1;
-	int skip_next_who:1;			/* used for "get my ip from server" */
+	int skip_next_userhost:1;			/* used for "get my ip from server" */
 	int inside_whois:1;
 	int doing_dns:1;				/* /dns has been done */
 	unsigned int end_of_motd:1;	/* end of motd reached (logged in) */
