@@ -255,6 +255,9 @@ gtkutil_file_req (char *title, void *callback, void *userdata, char *filter,
 		gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (dialog), TRUE);
 	if (last_dir[0])
 		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), last_dir);
+	if (flags & FRF_ADDFOLDER)
+		gtk_file_chooser_add_shortcut_folder (GTK_FILE_CHOOSER (dialog),
+														  (char *)get_xdir_fs (), NULL);
 
 	freq = malloc (sizeof (struct file_req));
 	freq->dialog = dialog;
