@@ -47,7 +47,9 @@
 #include "../common/text.h"
 #include "../common/xchatc.h"
 #include "../common/outbound.h"
+#include "../common/inbound.h"
 #include "../common/plugin.h"
+#include "../common/modes.h"
 #include "fe-gtk.h"
 #include "banlist.h"
 #include "gtkutil.h"
@@ -325,6 +327,16 @@ fe_set_title (session *sess)
 	gtk_window_set_title (GTK_WINDOW (sess->gui->window), tbuf);
 }
 
+static gboolean
+mg_windowstate_cb (GtkWidget *wid, GdkEvent *event, gpointer userdata)
+{
+	prefs.gui_win_state = 0;
+	if (gdk_window_get_state (wid->window) & GDK_WINDOW_STATE_MAXIMIZED)
+		prefs.gui_win_state = 1;
+
+	return FALSE;
+}
+
 static gboolean
 mg_configure_cb (GtkWidget *wid, GdkEventConfigure *event, session *sess)
 {
@@ -332,18 +344,13 @@ mg_configure_cb (GtkWidget *wid, GdkEventConfigure *event, session *sess)
 	{
 		if (mg_gui)
 		{
-/*			g_idle_add ((GSourceFunc)tab_group_resize, mg_gui->tabs_box);*/
 			if (prefs.mainwindow_save)
 			{
 				sess = current_sess;
 				gtk_window_get_position (GTK_WINDOW (wid), &prefs.mainwindow_left,
 												 &prefs.mainwindow_top);
 				gtk_window_get_size (GTK_WINDOW (wid), &prefs.mainwindow_width,
-												 &prefs.mainwindow_height);
-				/*prefs.mainwindow_left = event->x;
-				prefs.mainwindow_top = event->y;
-				prefs.mainwindow_width = event->width;
-				prefs.mainwindow_height = event->height;*/
+											&prefs.mainwindow_height);
 			}
 		}
 	}
@@ -356,10 +363,6 @@ mg_configure_cb (GtkWidget *wid, GdkEventConfigure *event, session *sess)
 											 &prefs.dialog_top);
 			gtk_window_get_size (GTK_WINDOW (wid), &prefs.dialog_width,
 										&prefs.dialog_height);
-			/*prefs.dialog_left = event->x;
-			prefs.dialog_top = event->y;
-			prefs.dialog_width = event->width;
-			prefs.dialog_height = event->height;*/
 		}
 
 		if (((GtkXText *) sess->gui->xtext)->transparent)
@@ -1404,7 +1407,7 @@ mg_change_flag (GtkWidget * wid, session *sess, char flag)
 			mode[0] = '+';
 		else
 			mode[0] = '-';
-		serv->p_chan_mode (serv, sess->channel, mode);
+		serv->p_mode (serv, sess->channel, mode);
 		serv->p_join_info (serv, sess->channel);
 		sess->ignore_mode = TRUE;
 		sess->ignore_date = TRUE;
@@ -1431,7 +1434,7 @@ flagl_hit (GtkWidget * wid, struct session *sess)
 				return;
 			}
 			snprintf (modes, sizeof (modes), "+l %d", atoi (limit_str));
-			serv->p_chan_mode (serv, sess->channel, modes);
+			serv->p_mode (serv, sess->channel, modes);
 			serv->p_join_info (serv, sess->channel);
 		}
 	} else
@@ -1452,7 +1455,7 @@ flagk_hit (GtkWidget * wid, struct session *sess)
 		if (GTK_TOGGLE_BUTTON (wid)->active)
 			modes[0] = '+';
 
-		serv->p_chan_mode (serv, sess->channel, modes);
+		serv->p_mode (serv, sess->channel, modes);
 	}
 }
 
@@ -1514,7 +1517,7 @@ mg_key_entry_cb (GtkWidget * igad, gpointer userdata)
 	{
 		snprintf (modes, sizeof (modes), "+k %s",
 				gtk_entry_get_text (GTK_ENTRY (igad)));
-		serv->p_chan_mode (serv, sess->channel, modes);
+		serv->p_mode (serv, sess->channel, modes);
 		serv->p_join_info (serv, sess->channel);
 	}
 }
@@ -1537,7 +1540,7 @@ mg_limit_entry_cb (GtkWidget * igad, gpointer userdata)
 		}
 		snprintf (modes, sizeof(modes), "+l %d", 
 				atoi (gtk_entry_get_text (GTK_ENTRY (igad))));
-		serv->p_chan_mode (serv, sess->channel, modes);
+		serv->p_mode (serv, sess->channel, modes);
 		serv->p_join_info (serv, sess->channel);
 	}
 }
@@ -1730,6 +1733,8 @@ mg_word_clicked (GtkWidget *xtext, char *word, GdkEventButton *even,
 	{
 		if (sess->type == SESS_DIALOG)
 			menu_middlemenu (sess, even);
+		else if (even->type == GDK_2BUTTON_PRESS)
+			userlist_select (sess, word);
 		return;
 	}
 
@@ -1777,6 +1782,7 @@ mg_update_xtext (GtkWidget *wid)
 	gtk_xtext_set_tint (xtext, prefs.tint_red, prefs.tint_green, prefs.tint_blue);
 	gtk_xtext_set_background (xtext, channelwin_pix, prefs.transparent, prefs.tint);
 	gtk_xtext_set_wordwrap (xtext, prefs.wordwrap);
+	gtk_xtext_set_show_marker (xtext, prefs.show_marker);
 	gtk_xtext_set_show_separator (xtext, prefs.indent_nicks ? prefs.show_separator : 0);
 	gtk_xtext_set_indent (xtext, prefs.indent_nicks);
 	if (!gtk_xtext_set_font (xtext, prefs.font_normal))
@@ -2125,6 +2131,7 @@ static gboolean
 mg_tabwin_focus_cb (GtkWindow * win, GdkEventFocus *event, gpointer userdata)
 {
 	current_sess = current_tab;
+	gtk_xtext_check_marker_visibility(GTK_XTEXT (current_sess->gui->xtext));
 	plugin_emit_dummy_print (current_sess, "Focus Window");
 	return FALSE;
 }
@@ -2135,6 +2142,7 @@ mg_topwin_focus_cb (GtkWindow * win, GdkEventFocus *event, session *sess)
 	current_sess = sess;
 	if (!sess->server->server_session)
 		sess->server->server_session = sess;
+	gtk_xtext_check_marker_visibility(GTK_XTEXT (current_sess->gui->xtext));
 	plugin_emit_dummy_print (sess, "Focus Window");
 	return FALSE;
 }
@@ -2305,8 +2313,8 @@ mg_create_tabwindow (session *sess)
 	sess->gui->window = win;
 	gtk_window_move (GTK_WINDOW (win), prefs.mainwindow_left,
 						  prefs.mainwindow_top);
-/*	gtk_window_set_default_size (GTK_WINDOW (win), prefs.mainwindow_width,
-										  prefs.mainwindow_height);*/
+	if (prefs.gui_win_state)
+		gtk_window_maximize (GTK_WINDOW (win));
 	gtk_container_set_border_width (GTK_CONTAINER (win), 2);
 
 	g_signal_connect (G_OBJECT (win), "delete_event",
@@ -2317,6 +2325,8 @@ mg_create_tabwindow (session *sess)
 							G_CALLBACK (mg_tabwin_focus_cb), NULL);
 	g_signal_connect (G_OBJECT (win), "configure_event",
 							G_CALLBACK (mg_configure_cb), NULL);
+	g_signal_connect (G_OBJECT (win), "window_state_event",
+							G_CALLBACK (mg_windowstate_cb), NULL);
 
 	palette_alloc (win);
 
@@ -2565,6 +2575,7 @@ mg_changui_new (session *sess, restore_gui *res, int tab)
 {
 	int first_run = FALSE;
 	session_gui *gui;
+	struct User *user = NULL;
 
 	if (!res)
 	{
@@ -2577,6 +2588,9 @@ mg_changui_new (session *sess, restore_gui *res, int tab)
 	if (!sess->server->front_session)
 		sess->server->front_session = sess;
 
+	if (!is_channel (sess->server, sess->channel))
+		user = find_name_global (sess->server, sess->channel);
+
 	if (!tab)
 	{
 		gui = malloc (sizeof (session_gui));
@@ -2585,6 +2599,8 @@ mg_changui_new (session *sess, restore_gui *res, int tab)
 		sess->gui = gui;
 		mg_create_topwindow (sess);
 		fe_set_title (sess);
+		if (user && user->hostname)
+			set_topic (sess, user->hostname);
 		return;
 	}
 
@@ -2604,6 +2620,9 @@ mg_changui_new (session *sess, restore_gui *res, int tab)
 		gui->is_tab = TRUE;
 	}
 
+	if (user && user->hostname)
+		set_topic (sess, user->hostname);
+
 	mg_add_chan (sess);
 
 	if (first_run || prefs.newtabstofront)
