@@ -114,6 +114,45 @@ fe_userlist_numbers (session *sess)
 	}
 }
 
+/* select a row in the userlist by nick-name */
+
+void
+userlist_select (session *sess, char *name)
+{
+	GtkTreeIter iter;
+	GtkTreePath *path;
+	GtkTreeView *treeview = GTK_TREE_VIEW (sess->gui->user_tree);
+	GtkTreeModel *model = gtk_tree_view_get_model (treeview);
+	GtkTreeSelection *selection = gtk_tree_view_get_selection (treeview);
+	struct User *row_user;
+
+	if (gtk_tree_model_get_iter_first (model, &iter))
+	{
+		do
+		{
+			gtk_tree_model_get (model, &iter, 3, &row_user, -1);
+			if (sess->server->p_cmp (row_user->nick, name) == 0)
+			{
+				if (gtk_tree_selection_iter_is_selected (selection, &iter))
+					gtk_tree_selection_unselect_iter (selection, &iter);
+				else
+					gtk_tree_selection_select_iter (selection, &iter);
+
+				/* and make sure it's visible */
+				path = gtk_tree_model_get_path (model, &iter);
+				if (path)
+				{
+					gtk_tree_view_scroll_to_cell (treeview, path, NULL, TRUE, 0.5, 0.5);
+					gtk_tree_path_free (path);
+				}
+
+				return;
+			}
+		}
+		while (gtk_tree_model_iter_next (model, &iter));
+	}
+}
+
 char **
 userlist_selection_list (GtkWidget *widget, int *num_ret)
 {
@@ -326,7 +365,7 @@ userlist_dnd_drop (GtkTreeView *widget, GdkDragContext *context,
 						 guint info, guint ttime, gpointer userdata)
 {
 	struct User *user;
-	char *p, *data, *next;
+	char *p, *data, *next, *fname;
 	GtkTreePath *path;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
@@ -347,12 +386,18 @@ userlist_dnd_drop (GtkTreeView *widget, GdkDragContext *context,
 		{
 			if (next)
 				*next = 0;
-			p += 5;
-#ifdef WIN32
-			if (strncmp (p, "///", 3) == 0)
-				p += 3;
-#endif
-			dcc_send (current_sess, user->nick, p, prefs.dcc_max_send_cps, 0);
+			fname = g_filename_from_uri (p, NULL, NULL);
+			if (fname)
+			{
+				/* dcc_send() expects utf-8 */
+				p = g_filename_to_utf8 (fname, -1, 0, 0, 0);
+				if (p)
+				{
+					dcc_send (current_sess, user->nick, p, prefs.dcc_max_send_cps, 0);
+					g_free (p);
+				}
+				g_free (fname);
+			}
 		}
 		if (!next)
 			break;
